# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_CAPTURE_PLANE_METADATA = Metadata(
    id="fcda0671173456c7a0dc9ad4eeb91facd1507c4f.workbench",
    name="volume-capture-plane",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeCapturePlaneParamsDictNoTag = typing.TypedDict('_VolumeCapturePlaneParamsDictNoTag', {
    "volume": InputPathType,
    "subvolume": str,
    "interp": str,
    "h-dim": int,
    "v-dim": int,
    "scale-min": float,
    "scale-max": float,
    "bottom-left-x": float,
    "bottom-left-y": float,
    "bottom-left-z": float,
    "bottom-right-x": float,
    "bottom-right-y": float,
    "bottom-right-z": float,
    "top-left-x": float,
    "top-left-y": float,
    "top-left-z": float,
    "image": str,
})
VolumeCapturePlaneParamsDictTagged = typing.TypedDict('VolumeCapturePlaneParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-capture-plane"],
    "volume": InputPathType,
    "subvolume": str,
    "interp": str,
    "h-dim": int,
    "v-dim": int,
    "scale-min": float,
    "scale-max": float,
    "bottom-left-x": float,
    "bottom-left-y": float,
    "bottom-left-z": float,
    "bottom-right-x": float,
    "bottom-right-y": float,
    "bottom-right-z": float,
    "top-left-x": float,
    "top-left-y": float,
    "top-left-z": float,
    "image": str,
})
VolumeCapturePlaneParamsDict = _VolumeCapturePlaneParamsDictNoTag | VolumeCapturePlaneParamsDictTagged


class VolumeCapturePlaneOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeCapturePlaneParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_capture_plane_params(
    volume: InputPathType,
    subvolume: str,
    interp: str,
    h_dim: int,
    v_dim: int,
    scale_min: float,
    scale_max: float,
    bottom_left_x: float,
    bottom_left_y: float,
    bottom_left_z: float,
    bottom_right_x: float,
    bottom_right_y: float,
    bottom_right_z: float,
    top_left_x: float,
    top_left_y: float,
    top_left_z: float,
    image: str,
) -> VolumeCapturePlaneParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the volume file to interpolate from.
        subvolume: the name or number of the subvolume to use.
        interp: interpolation type.
        h_dim: width of output image, in pixels.
        v_dim: height of output image, in pixels.
        scale_min: value to render as black.
        scale_max: value to render as white.
        bottom_left_x: x-coordinate of the bottom left of the output image.
        bottom_left_y: y-coordinate of the bottom left of the output image.
        bottom_left_z: z-coordinate of the bottom left of the output image.
        bottom_right_x: x-coordinate of the bottom right of the output image.
        bottom_right_y: y-coordinate of the bottom right of the output image.
        bottom_right_z: z-coordinate of the bottom right of the output image.
        top_left_x: x-coordinate of the top left of the output image.
        top_left_y: y-coordinate of the top left of the output image.
        top_left_z: z-coordinate of the top left of the output image.
        image: output - the output image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-capture-plane",
        "volume": volume,
        "subvolume": subvolume,
        "interp": interp,
        "h-dim": h_dim,
        "v-dim": v_dim,
        "scale-min": scale_min,
        "scale-max": scale_max,
        "bottom-left-x": bottom_left_x,
        "bottom-left-y": bottom_left_y,
        "bottom-left-z": bottom_left_z,
        "bottom-right-x": bottom_right_x,
        "bottom-right-y": bottom_right_y,
        "bottom-right-z": bottom_right_z,
        "top-left-x": top_left_x,
        "top-left-y": top_left_y,
        "top-left-z": top_left_z,
        "image": image,
    }
    return params


def volume_capture_plane_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeCapturePlaneParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("subvolume", None) is None:
        raise StyxValidationError("`subvolume` must not be None")
    if not isinstance(params["subvolume"], str):
        raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `str`')
    if params.get("interp", None) is None:
        raise StyxValidationError("`interp` must not be None")
    if not isinstance(params["interp"], str):
        raise StyxValidationError(f'`interp` has the wrong type: Received `{type(params.get("interp", None))}` expected `str`')
    if params.get("h-dim", None) is None:
        raise StyxValidationError("`h-dim` must not be None")
    if not isinstance(params["h-dim"], int):
        raise StyxValidationError(f'`h-dim` has the wrong type: Received `{type(params.get("h-dim", None))}` expected `int`')
    if params.get("v-dim", None) is None:
        raise StyxValidationError("`v-dim` must not be None")
    if not isinstance(params["v-dim"], int):
        raise StyxValidationError(f'`v-dim` has the wrong type: Received `{type(params.get("v-dim", None))}` expected `int`')
    if params.get("scale-min", None) is None:
        raise StyxValidationError("`scale-min` must not be None")
    if not isinstance(params["scale-min"], (float, int)):
        raise StyxValidationError(f'`scale-min` has the wrong type: Received `{type(params.get("scale-min", None))}` expected `float`')
    if params.get("scale-max", None) is None:
        raise StyxValidationError("`scale-max` must not be None")
    if not isinstance(params["scale-max"], (float, int)):
        raise StyxValidationError(f'`scale-max` has the wrong type: Received `{type(params.get("scale-max", None))}` expected `float`')
    if params.get("bottom-left-x", None) is None:
        raise StyxValidationError("`bottom-left-x` must not be None")
    if not isinstance(params["bottom-left-x"], (float, int)):
        raise StyxValidationError(f'`bottom-left-x` has the wrong type: Received `{type(params.get("bottom-left-x", None))}` expected `float`')
    if params.get("bottom-left-y", None) is None:
        raise StyxValidationError("`bottom-left-y` must not be None")
    if not isinstance(params["bottom-left-y"], (float, int)):
        raise StyxValidationError(f'`bottom-left-y` has the wrong type: Received `{type(params.get("bottom-left-y", None))}` expected `float`')
    if params.get("bottom-left-z", None) is None:
        raise StyxValidationError("`bottom-left-z` must not be None")
    if not isinstance(params["bottom-left-z"], (float, int)):
        raise StyxValidationError(f'`bottom-left-z` has the wrong type: Received `{type(params.get("bottom-left-z", None))}` expected `float`')
    if params.get("bottom-right-x", None) is None:
        raise StyxValidationError("`bottom-right-x` must not be None")
    if not isinstance(params["bottom-right-x"], (float, int)):
        raise StyxValidationError(f'`bottom-right-x` has the wrong type: Received `{type(params.get("bottom-right-x", None))}` expected `float`')
    if params.get("bottom-right-y", None) is None:
        raise StyxValidationError("`bottom-right-y` must not be None")
    if not isinstance(params["bottom-right-y"], (float, int)):
        raise StyxValidationError(f'`bottom-right-y` has the wrong type: Received `{type(params.get("bottom-right-y", None))}` expected `float`')
    if params.get("bottom-right-z", None) is None:
        raise StyxValidationError("`bottom-right-z` must not be None")
    if not isinstance(params["bottom-right-z"], (float, int)):
        raise StyxValidationError(f'`bottom-right-z` has the wrong type: Received `{type(params.get("bottom-right-z", None))}` expected `float`')
    if params.get("top-left-x", None) is None:
        raise StyxValidationError("`top-left-x` must not be None")
    if not isinstance(params["top-left-x"], (float, int)):
        raise StyxValidationError(f'`top-left-x` has the wrong type: Received `{type(params.get("top-left-x", None))}` expected `float`')
    if params.get("top-left-y", None) is None:
        raise StyxValidationError("`top-left-y` must not be None")
    if not isinstance(params["top-left-y"], (float, int)):
        raise StyxValidationError(f'`top-left-y` has the wrong type: Received `{type(params.get("top-left-y", None))}` expected `float`')
    if params.get("top-left-z", None) is None:
        raise StyxValidationError("`top-left-z` must not be None")
    if not isinstance(params["top-left-z"], (float, int)):
        raise StyxValidationError(f'`top-left-z` has the wrong type: Received `{type(params.get("top-left-z", None))}` expected `float`')
    if params.get("image", None) is None:
        raise StyxValidationError("`image` must not be None")
    if not isinstance(params["image"], str):
        raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `str`')


def volume_capture_plane_cargs(
    params: VolumeCapturePlaneParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-capture-plane"
    ])
    cargs.append(execution.input_file(params.get("volume", None)))
    cargs.append(params.get("subvolume", None))
    cargs.append(params.get("interp", None))
    cargs.append(str(params.get("h-dim", None)))
    cargs.append(str(params.get("v-dim", None)))
    cargs.append(str(params.get("scale-min", None)))
    cargs.append(str(params.get("scale-max", None)))
    cargs.append(str(params.get("bottom-left-x", None)))
    cargs.append(str(params.get("bottom-left-y", None)))
    cargs.append(str(params.get("bottom-left-z", None)))
    cargs.append(str(params.get("bottom-right-x", None)))
    cargs.append(str(params.get("bottom-right-y", None)))
    cargs.append(str(params.get("bottom-right-z", None)))
    cargs.append(str(params.get("top-left-x", None)))
    cargs.append(str(params.get("top-left-y", None)))
    cargs.append(str(params.get("top-left-z", None)))
    cargs.append(params.get("image", None))
    return cargs


def volume_capture_plane_outputs(
    params: VolumeCapturePlaneParamsDict,
    execution: Execution,
) -> VolumeCapturePlaneOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeCapturePlaneOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_capture_plane_execute(
    params: VolumeCapturePlaneParamsDict,
    runner: Runner | None = None,
) -> VolumeCapturePlaneOutputs:
    """
    INTERPOLATE IMAGE FROM PLANE THROUGH VOLUME.
    
    NOTE: If you want to generate an image with all of the capabilities of the
    GUI rendering, see -show-scene.
    
    Renders an image of an arbitrary plane through the volume file, with a
    simple linear grayscale palette. The parameter <interp> must be one of:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeCapturePlaneOutputs`).
    """
    volume_capture_plane_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_CAPTURE_PLANE_METADATA)
    params = execution.params(params)
    cargs = volume_capture_plane_cargs(params, execution)
    ret = volume_capture_plane_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_capture_plane(
    volume: InputPathType,
    subvolume: str,
    interp: str,
    h_dim: int,
    v_dim: int,
    scale_min: float,
    scale_max: float,
    bottom_left_x: float,
    bottom_left_y: float,
    bottom_left_z: float,
    bottom_right_x: float,
    bottom_right_y: float,
    bottom_right_z: float,
    top_left_x: float,
    top_left_y: float,
    top_left_z: float,
    image: str,
    runner: Runner | None = None,
) -> VolumeCapturePlaneOutputs:
    """
    INTERPOLATE IMAGE FROM PLANE THROUGH VOLUME.
    
    NOTE: If you want to generate an image with all of the capabilities of the
    GUI rendering, see -show-scene.
    
    Renders an image of an arbitrary plane through the volume file, with a
    simple linear grayscale palette. The parameter <interp> must be one of:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR.
    
    Args:
        volume: the volume file to interpolate from.
        subvolume: the name or number of the subvolume to use.
        interp: interpolation type.
        h_dim: width of output image, in pixels.
        v_dim: height of output image, in pixels.
        scale_min: value to render as black.
        scale_max: value to render as white.
        bottom_left_x: x-coordinate of the bottom left of the output image.
        bottom_left_y: y-coordinate of the bottom left of the output image.
        bottom_left_z: z-coordinate of the bottom left of the output image.
        bottom_right_x: x-coordinate of the bottom right of the output image.
        bottom_right_y: y-coordinate of the bottom right of the output image.
        bottom_right_z: z-coordinate of the bottom right of the output image.
        top_left_x: x-coordinate of the top left of the output image.
        top_left_y: y-coordinate of the top left of the output image.
        top_left_z: z-coordinate of the top left of the output image.
        image: output - the output image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeCapturePlaneOutputs`).
    """
    params = volume_capture_plane_params(
        volume=volume,
        subvolume=subvolume,
        interp=interp,
        h_dim=h_dim,
        v_dim=v_dim,
        scale_min=scale_min,
        scale_max=scale_max,
        bottom_left_x=bottom_left_x,
        bottom_left_y=bottom_left_y,
        bottom_left_z=bottom_left_z,
        bottom_right_x=bottom_right_x,
        bottom_right_y=bottom_right_y,
        bottom_right_z=bottom_right_z,
        top_left_x=top_left_x,
        top_left_y=top_left_y,
        top_left_z=top_left_z,
        image=image,
    )
    return volume_capture_plane_execute(params, runner)


__all__ = [
    "VOLUME_CAPTURE_PLANE_METADATA",
    "VolumeCapturePlaneOutputs",
    "VolumeCapturePlaneParamsDict",
    "VolumeCapturePlaneParamsDictTagged",
    "volume_capture_plane",
    "volume_capture_plane_execute",
    "volume_capture_plane_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CONVERT_METADATA = Metadata(
    id="ab14ca28df7d909cf2bd3b8aa98a47d8e0cd6a2c.workbench",
    name="cifti-convert",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiConvertToGiftiExtParamsDictNoTag = typing.TypedDict('_CiftiConvertToGiftiExtParamsDictNoTag', {
    "cifti-in": InputPathType,
    "gifti-out": str,
})
CiftiConvertToGiftiExtParamsDictTagged = typing.TypedDict('CiftiConvertToGiftiExtParamsDictTagged', {
    "@type": typing.Literal["to-gifti-ext"],
    "cifti-in": InputPathType,
    "gifti-out": str,
})
CiftiConvertToGiftiExtParamsDict = _CiftiConvertToGiftiExtParamsDictNoTag | CiftiConvertToGiftiExtParamsDictTagged


_CiftiConvertResetTimepointsParamsDictNoTag = typing.TypedDict('_CiftiConvertResetTimepointsParamsDictNoTag', {
    "timestep": float,
    "timestart": float,
    "unit": typing.NotRequired[str | None],
})
CiftiConvertResetTimepointsParamsDictTagged = typing.TypedDict('CiftiConvertResetTimepointsParamsDictTagged', {
    "@type": typing.Literal["reset-timepoints"],
    "timestep": float,
    "timestart": float,
    "unit": typing.NotRequired[str | None],
})
CiftiConvertResetTimepointsParamsDict = _CiftiConvertResetTimepointsParamsDictNoTag | CiftiConvertResetTimepointsParamsDictTagged


_CiftiConvertReplaceBinaryParamsDictNoTag = typing.TypedDict('_CiftiConvertReplaceBinaryParamsDictNoTag', {
    "binary-in": str,
    "transpose": bool,
    "flip-endian": bool,
})
CiftiConvertReplaceBinaryParamsDictTagged = typing.TypedDict('CiftiConvertReplaceBinaryParamsDictTagged', {
    "@type": typing.Literal["replace-binary"],
    "binary-in": str,
    "transpose": bool,
    "flip-endian": bool,
})
CiftiConvertReplaceBinaryParamsDict = _CiftiConvertReplaceBinaryParamsDictNoTag | CiftiConvertReplaceBinaryParamsDictTagged


_CiftiConvertFromGiftiExtParamsDictNoTag = typing.TypedDict('_CiftiConvertFromGiftiExtParamsDictNoTag', {
    "gifti-in": str,
    "cifti-out": str,
    "reset-timepoints": typing.NotRequired[CiftiConvertResetTimepointsParamsDict | None],
    "replace-binary": typing.NotRequired[CiftiConvertReplaceBinaryParamsDict | None],
    "column-reset-scalars": bool,
    "reset-scalars": bool,
})
CiftiConvertFromGiftiExtParamsDictTagged = typing.TypedDict('CiftiConvertFromGiftiExtParamsDictTagged', {
    "@type": typing.Literal["from-gifti-ext"],
    "gifti-in": str,
    "cifti-out": str,
    "reset-timepoints": typing.NotRequired[CiftiConvertResetTimepointsParamsDict | None],
    "replace-binary": typing.NotRequired[CiftiConvertReplaceBinaryParamsDict | None],
    "column-reset-scalars": bool,
    "reset-scalars": bool,
})
CiftiConvertFromGiftiExtParamsDict = _CiftiConvertFromGiftiExtParamsDictNoTag | CiftiConvertFromGiftiExtParamsDictTagged


_CiftiConvertToNiftiParamsDictNoTag = typing.TypedDict('_CiftiConvertToNiftiParamsDictNoTag', {
    "cifti-in": InputPathType,
    "nifti-out": str,
    "smaller-dims": bool,
    "smaller-file": bool,
})
CiftiConvertToNiftiParamsDictTagged = typing.TypedDict('CiftiConvertToNiftiParamsDictTagged', {
    "@type": typing.Literal["to-nifti"],
    "cifti-in": InputPathType,
    "nifti-out": str,
    "smaller-dims": bool,
    "smaller-file": bool,
})
CiftiConvertToNiftiParamsDict = _CiftiConvertToNiftiParamsDictNoTag | CiftiConvertToNiftiParamsDictTagged


_CiftiConvertResetTimepointsParamsDict_NoTag = typing.TypedDict('_CiftiConvertResetTimepointsParamsDict_NoTag', {
    "timestep": float,
    "timestart": float,
    "unit": typing.NotRequired[str | None],
})
CiftiConvertResetTimepointsParamsDictTagged_ = typing.TypedDict('CiftiConvertResetTimepointsParamsDictTagged_', {
    "@type": typing.Literal["reset-timepoints"],
    "timestep": float,
    "timestart": float,
    "unit": typing.NotRequired[str | None],
})
CiftiConvertResetTimepointsParamsDict_ = _CiftiConvertResetTimepointsParamsDict_NoTag | CiftiConvertResetTimepointsParamsDictTagged_


_CiftiConvertFromNiftiParamsDictNoTag = typing.TypedDict('_CiftiConvertFromNiftiParamsDictNoTag', {
    "nifti-in": InputPathType,
    "cifti-template": InputPathType,
    "cifti-out": str,
    "reset-timepoints": typing.NotRequired[CiftiConvertResetTimepointsParamsDict_ | None],
    "reset-scalars": bool,
})
CiftiConvertFromNiftiParamsDictTagged = typing.TypedDict('CiftiConvertFromNiftiParamsDictTagged', {
    "@type": typing.Literal["from-nifti"],
    "nifti-in": InputPathType,
    "cifti-template": InputPathType,
    "cifti-out": str,
    "reset-timepoints": typing.NotRequired[CiftiConvertResetTimepointsParamsDict_ | None],
    "reset-scalars": bool,
})
CiftiConvertFromNiftiParamsDict = _CiftiConvertFromNiftiParamsDictNoTag | CiftiConvertFromNiftiParamsDictTagged


_CiftiConvertToTextParamsDictNoTag = typing.TypedDict('_CiftiConvertToTextParamsDictNoTag', {
    "cifti-in": InputPathType,
    "text-out": str,
    "delim-string": typing.NotRequired[str | None],
})
CiftiConvertToTextParamsDictTagged = typing.TypedDict('CiftiConvertToTextParamsDictTagged', {
    "@type": typing.Literal["to-text"],
    "cifti-in": InputPathType,
    "text-out": str,
    "delim-string": typing.NotRequired[str | None],
})
CiftiConvertToTextParamsDict = _CiftiConvertToTextParamsDictNoTag | CiftiConvertToTextParamsDictTagged


_CiftiConvertResetTimepointsParamsDict_2NoTag = typing.TypedDict('_CiftiConvertResetTimepointsParamsDict_2NoTag', {
    "timestep": float,
    "timestart": float,
    "unit": typing.NotRequired[str | None],
})
CiftiConvertResetTimepointsParamsDictTagged_2 = typing.TypedDict('CiftiConvertResetTimepointsParamsDictTagged_2', {
    "@type": typing.Literal["reset-timepoints"],
    "timestep": float,
    "timestart": float,
    "unit": typing.NotRequired[str | None],
})
CiftiConvertResetTimepointsParamsDict_2 = _CiftiConvertResetTimepointsParamsDict_2NoTag | CiftiConvertResetTimepointsParamsDictTagged_2


_CiftiConvertFromTextParamsDictNoTag = typing.TypedDict('_CiftiConvertFromTextParamsDictNoTag', {
    "text-in": str,
    "cifti-template": InputPathType,
    "cifti-out": str,
    "reset-timepoints": typing.NotRequired[CiftiConvertResetTimepointsParamsDict_2 | None],
    "delim-string": typing.NotRequired[str | None],
    "reset-scalars": bool,
})
CiftiConvertFromTextParamsDictTagged = typing.TypedDict('CiftiConvertFromTextParamsDictTagged', {
    "@type": typing.Literal["from-text"],
    "text-in": str,
    "cifti-template": InputPathType,
    "cifti-out": str,
    "reset-timepoints": typing.NotRequired[CiftiConvertResetTimepointsParamsDict_2 | None],
    "delim-string": typing.NotRequired[str | None],
    "reset-scalars": bool,
})
CiftiConvertFromTextParamsDict = _CiftiConvertFromTextParamsDictNoTag | CiftiConvertFromTextParamsDictTagged


_CiftiConvertParamsDictNoTag = typing.TypedDict('_CiftiConvertParamsDictNoTag', {
    "to-gifti-ext": typing.NotRequired[CiftiConvertToGiftiExtParamsDict | None],
    "from-gifti-ext": typing.NotRequired[CiftiConvertFromGiftiExtParamsDict | None],
    "to-nifti": typing.NotRequired[CiftiConvertToNiftiParamsDict | None],
    "from-nifti": typing.NotRequired[CiftiConvertFromNiftiParamsDict | None],
    "to-text": typing.NotRequired[CiftiConvertToTextParamsDict | None],
    "from-text": typing.NotRequired[CiftiConvertFromTextParamsDict | None],
})
CiftiConvertParamsDictTagged = typing.TypedDict('CiftiConvertParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-convert"],
    "to-gifti-ext": typing.NotRequired[CiftiConvertToGiftiExtParamsDict | None],
    "from-gifti-ext": typing.NotRequired[CiftiConvertFromGiftiExtParamsDict | None],
    "to-nifti": typing.NotRequired[CiftiConvertToNiftiParamsDict | None],
    "from-nifti": typing.NotRequired[CiftiConvertFromNiftiParamsDict | None],
    "to-text": typing.NotRequired[CiftiConvertToTextParamsDict | None],
    "from-text": typing.NotRequired[CiftiConvertFromTextParamsDict | None],
})
CiftiConvertParamsDict = _CiftiConvertParamsDictNoTag | CiftiConvertParamsDictTagged


def cifti_convert_to_gifti_ext(
    cifti_in: InputPathType,
    gifti_out: str,
) -> CiftiConvertToGiftiExtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        gifti_out: output - the output gifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "to-gifti-ext",
        "cifti-in": cifti_in,
        "gifti-out": gifti_out,
    }
    return params


def cifti_convert_to_gifti_ext_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertToGiftiExtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("gifti-out", None) is None:
        raise StyxValidationError("`gifti-out` must not be None")
    if not isinstance(params["gifti-out"], str):
        raise StyxValidationError(f'`gifti-out` has the wrong type: Received `{type(params.get("gifti-out", None))}` expected `str`')


def cifti_convert_to_gifti_ext_cargs(
    params: CiftiConvertToGiftiExtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-to-gifti-ext",
        execution.input_file(params.get("cifti-in", None)),
        params.get("gifti-out", None)
    ])
    return cargs


def cifti_convert_reset_timepoints(
    timestep: float,
    timestart: float,
    unit: str | None = None,
) -> CiftiConvertResetTimepointsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        timestep: the desired time between frames.
        timestart: the desired time offset of the initial frame.
        unit: use a unit other than time\
            \
            unit identifier (default SECOND).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "reset-timepoints",
        "timestep": timestep,
        "timestart": timestart,
    }
    if unit is not None:
        params["unit"] = unit
    return params


def cifti_convert_reset_timepoints_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertResetTimepointsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("timestep", None) is None:
        raise StyxValidationError("`timestep` must not be None")
    if not isinstance(params["timestep"], (float, int)):
        raise StyxValidationError(f'`timestep` has the wrong type: Received `{type(params.get("timestep", None))}` expected `float`')
    if params.get("timestart", None) is None:
        raise StyxValidationError("`timestart` must not be None")
    if not isinstance(params["timestart"], (float, int)):
        raise StyxValidationError(f'`timestart` has the wrong type: Received `{type(params.get("timestart", None))}` expected `float`')
    if params.get("unit", None) is not None:
        if not isinstance(params["unit"], str):
            raise StyxValidationError(f'`unit` has the wrong type: Received `{type(params.get("unit", None))}` expected `str | None`')


def cifti_convert_reset_timepoints_cargs(
    params: CiftiConvertResetTimepointsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-reset-timepoints",
        str(params.get("timestep", None)),
        str(params.get("timestart", None))
    ])
    if params.get("unit", None) is not None:
        cargs.extend([
            "-unit",
            params.get("unit", None)
        ])
    return cargs


def cifti_convert_replace_binary(
    binary_in: str,
    transpose: bool = False,
    flip_endian: bool = False,
) -> CiftiConvertReplaceBinaryParamsDictTagged:
    """
    Build parameters.
    
    Args:
        binary_in: the binary file that contains replacement data.
        transpose: transpose the binary file.
        flip_endian: byteswap the binary file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "replace-binary",
        "binary-in": binary_in,
        "transpose": transpose,
        "flip-endian": flip_endian,
    }
    return params


def cifti_convert_replace_binary_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertReplaceBinaryParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("binary-in", None) is None:
        raise StyxValidationError("`binary-in` must not be None")
    if not isinstance(params["binary-in"], str):
        raise StyxValidationError(f'`binary-in` has the wrong type: Received `{type(params.get("binary-in", None))}` expected `str`')
    if params.get("transpose", False) is None:
        raise StyxValidationError("`transpose` must not be None")
    if not isinstance(params["transpose"], bool):
        raise StyxValidationError(f'`transpose` has the wrong type: Received `{type(params.get("transpose", False))}` expected `bool`')
    if params.get("flip-endian", False) is None:
        raise StyxValidationError("`flip-endian` must not be None")
    if not isinstance(params["flip-endian"], bool):
        raise StyxValidationError(f'`flip-endian` has the wrong type: Received `{type(params.get("flip-endian", False))}` expected `bool`')


def cifti_convert_replace_binary_cargs(
    params: CiftiConvertReplaceBinaryParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-replace-binary",
        params.get("binary-in", None)
    ])
    if params.get("transpose", False):
        cargs.append("-transpose")
    if params.get("flip-endian", False):
        cargs.append("-flip-endian")
    return cargs


class CiftiConvertFromGiftiExtOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertFromGiftiExtParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_convert_from_gifti_ext(
    gifti_in: str,
    cifti_out: str,
    reset_timepoints: CiftiConvertResetTimepointsParamsDict | None = None,
    replace_binary: CiftiConvertReplaceBinaryParamsDict | None = None,
    column_reset_scalars: bool = False,
    reset_scalars: bool = False,
) -> CiftiConvertFromGiftiExtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        gifti_in: the input gifti file.
        cifti_out: the output cifti file.
        reset_timepoints: reset the mapping along rows to timepoints, taking\
            length from the gifti file.
        replace_binary: replace data with a binary file.
        column_reset_scalars: reset mapping along columns to scalar (useful for\
            changing number of series in a sdseries file).
        reset_scalars: reset mapping along rows to scalars, taking length from\
            the gifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-gifti-ext",
        "gifti-in": gifti_in,
        "cifti-out": cifti_out,
        "column-reset-scalars": column_reset_scalars,
        "reset-scalars": reset_scalars,
    }
    if reset_timepoints is not None:
        params["reset-timepoints"] = reset_timepoints
    if replace_binary is not None:
        params["replace-binary"] = replace_binary
    return params


def cifti_convert_from_gifti_ext_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertFromGiftiExtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("gifti-in", None) is None:
        raise StyxValidationError("`gifti-in` must not be None")
    if not isinstance(params["gifti-in"], str):
        raise StyxValidationError(f'`gifti-in` has the wrong type: Received `{type(params.get("gifti-in", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("reset-timepoints", None) is not None:
        cifti_convert_reset_timepoints_validate(params["reset-timepoints"])
    if params.get("replace-binary", None) is not None:
        cifti_convert_replace_binary_validate(params["replace-binary"])
    if params.get("column-reset-scalars", False) is None:
        raise StyxValidationError("`column-reset-scalars` must not be None")
    if not isinstance(params["column-reset-scalars"], bool):
        raise StyxValidationError(f'`column-reset-scalars` has the wrong type: Received `{type(params.get("column-reset-scalars", False))}` expected `bool`')
    if params.get("reset-scalars", False) is None:
        raise StyxValidationError("`reset-scalars` must not be None")
    if not isinstance(params["reset-scalars"], bool):
        raise StyxValidationError(f'`reset-scalars` has the wrong type: Received `{type(params.get("reset-scalars", False))}` expected `bool`')


def cifti_convert_from_gifti_ext_cargs(
    params: CiftiConvertFromGiftiExtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-gifti-ext",
        params.get("gifti-in", None),
        params.get("cifti-out", None),
        *(cifti_convert_reset_timepoints_cargs(params.get("reset-timepoints", None), execution) if (params.get("reset-timepoints", None) is not None) else []),
        *(cifti_convert_replace_binary_cargs(params.get("replace-binary", None), execution) if (params.get("replace-binary", None) is not None) else [])
    ])
    if params.get("column-reset-scalars", False):
        cargs.append("-column-reset-scalars")
    if params.get("reset-scalars", False):
        cargs.append("-reset-scalars")
    return cargs


def cifti_convert_from_gifti_ext_outputs(
    params: CiftiConvertFromGiftiExtParamsDict,
    execution: Execution,
) -> CiftiConvertFromGiftiExtOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiConvertFromGiftiExtOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


class CiftiConvertToNiftiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertToNiftiParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    nifti_out: OutputPathType
    """the output nifti file"""


def cifti_convert_to_nifti(
    cifti_in: InputPathType,
    nifti_out: str,
    smaller_dims: bool = False,
    smaller_file: bool = False,
) -> CiftiConvertToNiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        nifti_out: the output nifti file.
        smaller_dims: minimize the largest dimension, for tools that don't like\
            large indices.
        smaller_file: use better-fitting dimension lengths.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "to-nifti",
        "cifti-in": cifti_in,
        "nifti-out": nifti_out,
        "smaller-dims": smaller_dims,
        "smaller-file": smaller_file,
    }
    return params


def cifti_convert_to_nifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertToNiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("nifti-out", None) is None:
        raise StyxValidationError("`nifti-out` must not be None")
    if not isinstance(params["nifti-out"], str):
        raise StyxValidationError(f'`nifti-out` has the wrong type: Received `{type(params.get("nifti-out", None))}` expected `str`')
    if params.get("smaller-dims", False) is None:
        raise StyxValidationError("`smaller-dims` must not be None")
    if not isinstance(params["smaller-dims"], bool):
        raise StyxValidationError(f'`smaller-dims` has the wrong type: Received `{type(params.get("smaller-dims", False))}` expected `bool`')
    if params.get("smaller-file", False) is None:
        raise StyxValidationError("`smaller-file` must not be None")
    if not isinstance(params["smaller-file"], bool):
        raise StyxValidationError(f'`smaller-file` has the wrong type: Received `{type(params.get("smaller-file", False))}` expected `bool`')


def cifti_convert_to_nifti_cargs(
    params: CiftiConvertToNiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-to-nifti",
        execution.input_file(params.get("cifti-in", None)),
        params.get("nifti-out", None)
    ])
    if params.get("smaller-dims", False):
        cargs.append("-smaller-dims")
    if params.get("smaller-file", False):
        cargs.append("-smaller-file")
    return cargs


def cifti_convert_to_nifti_outputs(
    params: CiftiConvertToNiftiParamsDict,
    execution: Execution,
) -> CiftiConvertToNiftiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiConvertToNiftiOutputs(
        root=execution.output_file("."),
        nifti_out=execution.output_file(params.get("nifti-out", None)),
    )
    return ret


def cifti_convert_reset_timepoints_(
    timestep: float,
    timestart: float,
    unit: str | None = None,
) -> CiftiConvertResetTimepointsParamsDictTagged_:
    """
    Build parameters.
    
    Args:
        timestep: the desired time between frames.
        timestart: the desired time offset of the initial frame.
        unit: use a unit other than time\
            \
            unit identifier (default SECOND).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "reset-timepoints",
        "timestep": timestep,
        "timestart": timestart,
    }
    if unit is not None:
        params["unit"] = unit
    return params


def cifti_convert_reset_timepoints_validate_(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertResetTimepointsParamsDict_` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("timestep", None) is None:
        raise StyxValidationError("`timestep` must not be None")
    if not isinstance(params["timestep"], (float, int)):
        raise StyxValidationError(f'`timestep` has the wrong type: Received `{type(params.get("timestep", None))}` expected `float`')
    if params.get("timestart", None) is None:
        raise StyxValidationError("`timestart` must not be None")
    if not isinstance(params["timestart"], (float, int)):
        raise StyxValidationError(f'`timestart` has the wrong type: Received `{type(params.get("timestart", None))}` expected `float`')
    if params.get("unit", None) is not None:
        if not isinstance(params["unit"], str):
            raise StyxValidationError(f'`unit` has the wrong type: Received `{type(params.get("unit", None))}` expected `str | None`')


def cifti_convert_reset_timepoints_cargs_(
    params: CiftiConvertResetTimepointsParamsDict_,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-reset-timepoints",
        str(params.get("timestep", None)),
        str(params.get("timestart", None))
    ])
    if params.get("unit", None) is not None:
        cargs.extend([
            "-unit",
            params.get("unit", None)
        ])
    return cargs


class CiftiConvertFromNiftiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertFromNiftiParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_convert_from_nifti(
    nifti_in: InputPathType,
    cifti_template: InputPathType,
    cifti_out: str,
    reset_timepoints: CiftiConvertResetTimepointsParamsDict_ | None = None,
    reset_scalars: bool = False,
) -> CiftiConvertFromNiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        nifti_in: the input nifti file.
        cifti_template: a cifti file with the dimension(s) and mapping(s) that\
            should be used.
        cifti_out: the output cifti file.
        reset_timepoints: reset the mapping along rows to timepoints, taking\
            length from the nifti file.
        reset_scalars: reset mapping along rows to scalars, taking length from\
            the nifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-nifti",
        "nifti-in": nifti_in,
        "cifti-template": cifti_template,
        "cifti-out": cifti_out,
        "reset-scalars": reset_scalars,
    }
    if reset_timepoints is not None:
        params["reset-timepoints"] = reset_timepoints
    return params


def cifti_convert_from_nifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertFromNiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("nifti-in", None) is None:
        raise StyxValidationError("`nifti-in` must not be None")
    if not isinstance(params["nifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`nifti-in` has the wrong type: Received `{type(params.get("nifti-in", None))}` expected `InputPathType`')
    if params.get("cifti-template", None) is None:
        raise StyxValidationError("`cifti-template` must not be None")
    if not isinstance(params["cifti-template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-template` has the wrong type: Received `{type(params.get("cifti-template", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("reset-timepoints", None) is not None:
        cifti_convert_reset_timepoints_validate_(params["reset-timepoints"])
    if params.get("reset-scalars", False) is None:
        raise StyxValidationError("`reset-scalars` must not be None")
    if not isinstance(params["reset-scalars"], bool):
        raise StyxValidationError(f'`reset-scalars` has the wrong type: Received `{type(params.get("reset-scalars", False))}` expected `bool`')


def cifti_convert_from_nifti_cargs(
    params: CiftiConvertFromNiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-nifti",
        execution.input_file(params.get("nifti-in", None)),
        execution.input_file(params.get("cifti-template", None)),
        params.get("cifti-out", None),
        *(cifti_convert_reset_timepoints_cargs_(params.get("reset-timepoints", None), execution) if (params.get("reset-timepoints", None) is not None) else [])
    ])
    if params.get("reset-scalars", False):
        cargs.append("-reset-scalars")
    return cargs


def cifti_convert_from_nifti_outputs(
    params: CiftiConvertFromNiftiParamsDict,
    execution: Execution,
) -> CiftiConvertFromNiftiOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiConvertFromNiftiOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_convert_to_text(
    cifti_in: InputPathType,
    text_out: str,
    delim_string: str | None = None,
) -> CiftiConvertToTextParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        text_out: output - the output text file.
        delim_string: choose string to put between elements in a row\
            \
            the string to use (default is a tab character).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "to-text",
        "cifti-in": cifti_in,
        "text-out": text_out,
    }
    if delim_string is not None:
        params["delim-string"] = delim_string
    return params


def cifti_convert_to_text_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertToTextParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("text-out", None) is None:
        raise StyxValidationError("`text-out` must not be None")
    if not isinstance(params["text-out"], str):
        raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str`')
    if params.get("delim-string", None) is not None:
        if not isinstance(params["delim-string"], str):
            raise StyxValidationError(f'`delim-string` has the wrong type: Received `{type(params.get("delim-string", None))}` expected `str | None`')


def cifti_convert_to_text_cargs(
    params: CiftiConvertToTextParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-to-text",
        execution.input_file(params.get("cifti-in", None)),
        params.get("text-out", None)
    ])
    if params.get("delim-string", None) is not None:
        cargs.extend([
            "-col-delim",
            params.get("delim-string", None)
        ])
    return cargs


def cifti_convert_reset_timepoints_2(
    timestep: float,
    timestart: float,
    unit: str | None = None,
) -> CiftiConvertResetTimepointsParamsDictTagged_2:
    """
    Build parameters.
    
    Args:
        timestep: the desired time between frames.
        timestart: the desired time offset of the initial frame.
        unit: use a unit other than time\
            \
            unit identifier (default SECOND).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "reset-timepoints",
        "timestep": timestep,
        "timestart": timestart,
    }
    if unit is not None:
        params["unit"] = unit
    return params


def cifti_convert_reset_timepoints_validate_2(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertResetTimepointsParamsDict_2` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("timestep", None) is None:
        raise StyxValidationError("`timestep` must not be None")
    if not isinstance(params["timestep"], (float, int)):
        raise StyxValidationError(f'`timestep` has the wrong type: Received `{type(params.get("timestep", None))}` expected `float`')
    if params.get("timestart", None) is None:
        raise StyxValidationError("`timestart` must not be None")
    if not isinstance(params["timestart"], (float, int)):
        raise StyxValidationError(f'`timestart` has the wrong type: Received `{type(params.get("timestart", None))}` expected `float`')
    if params.get("unit", None) is not None:
        if not isinstance(params["unit"], str):
            raise StyxValidationError(f'`unit` has the wrong type: Received `{type(params.get("unit", None))}` expected `str | None`')


def cifti_convert_reset_timepoints_cargs_2(
    params: CiftiConvertResetTimepointsParamsDict_2,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-reset-timepoints",
        str(params.get("timestep", None)),
        str(params.get("timestart", None))
    ])
    if params.get("unit", None) is not None:
        cargs.extend([
            "-unit",
            params.get("unit", None)
        ])
    return cargs


class CiftiConvertFromTextOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertFromTextParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_convert_from_text(
    text_in: str,
    cifti_template: InputPathType,
    cifti_out: str,
    reset_timepoints: CiftiConvertResetTimepointsParamsDict_2 | None = None,
    delim_string: str | None = None,
    reset_scalars: bool = False,
) -> CiftiConvertFromTextParamsDictTagged:
    """
    Build parameters.
    
    Args:
        text_in: the input text file.
        cifti_template: a cifti file with the dimension(s) and mapping(s) that\
            should be used.
        cifti_out: the output cifti file.
        reset_timepoints: reset the mapping along rows to timepoints, taking\
            length from the text file.
        delim_string: specify string that is between elements in a row\
            \
            the string to use (default is any whitespace).
        reset_scalars: reset mapping along rows to scalars, taking length from\
            the text file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-text",
        "text-in": text_in,
        "cifti-template": cifti_template,
        "cifti-out": cifti_out,
        "reset-scalars": reset_scalars,
    }
    if reset_timepoints is not None:
        params["reset-timepoints"] = reset_timepoints
    if delim_string is not None:
        params["delim-string"] = delim_string
    return params


def cifti_convert_from_text_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertFromTextParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("text-in", None) is None:
        raise StyxValidationError("`text-in` must not be None")
    if not isinstance(params["text-in"], str):
        raise StyxValidationError(f'`text-in` has the wrong type: Received `{type(params.get("text-in", None))}` expected `str`')
    if params.get("cifti-template", None) is None:
        raise StyxValidationError("`cifti-template` must not be None")
    if not isinstance(params["cifti-template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-template` has the wrong type: Received `{type(params.get("cifti-template", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("reset-timepoints", None) is not None:
        cifti_convert_reset_timepoints_validate_2(params["reset-timepoints"])
    if params.get("delim-string", None) is not None:
        if not isinstance(params["delim-string"], str):
            raise StyxValidationError(f'`delim-string` has the wrong type: Received `{type(params.get("delim-string", None))}` expected `str | None`')
    if params.get("reset-scalars", False) is None:
        raise StyxValidationError("`reset-scalars` must not be None")
    if not isinstance(params["reset-scalars"], bool):
        raise StyxValidationError(f'`reset-scalars` has the wrong type: Received `{type(params.get("reset-scalars", False))}` expected `bool`')


def cifti_convert_from_text_cargs(
    params: CiftiConvertFromTextParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-text",
        params.get("text-in", None),
        execution.input_file(params.get("cifti-template", None)),
        params.get("cifti-out", None),
        *(cifti_convert_reset_timepoints_cargs_2(params.get("reset-timepoints", None), execution) if (params.get("reset-timepoints", None) is not None) else [])
    ])
    if params.get("delim-string", None) is not None:
        cargs.extend([
            "-col-delim",
            params.get("delim-string", None)
        ])
    if params.get("reset-scalars", False):
        cargs.append("-reset-scalars")
    return cargs


def cifti_convert_from_text_outputs(
    params: CiftiConvertFromTextParamsDict,
    execution: Execution,
) -> CiftiConvertFromTextOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiConvertFromTextOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


class CiftiConvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiConvertParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    from_gifti_ext: CiftiConvertFromGiftiExtOutputs | None
    """Outputs from `cifti_convert_from_gifti_ext_outputs`."""
    to_nifti: CiftiConvertToNiftiOutputs | None
    """Outputs from `cifti_convert_to_nifti_outputs`."""
    from_nifti: CiftiConvertFromNiftiOutputs | None
    """Outputs from `cifti_convert_from_nifti_outputs`."""
    from_text: CiftiConvertFromTextOutputs | None
    """Outputs from `cifti_convert_from_text_outputs`."""


def cifti_convert_params(
    to_gifti_ext: CiftiConvertToGiftiExtParamsDict | None = None,
    from_gifti_ext: CiftiConvertFromGiftiExtParamsDict | None = None,
    to_nifti: CiftiConvertToNiftiParamsDict | None = None,
    from_nifti: CiftiConvertFromNiftiParamsDict | None = None,
    to_text: CiftiConvertToTextParamsDict | None = None,
    from_text: CiftiConvertFromTextParamsDict | None = None,
) -> CiftiConvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        to_gifti_ext: convert to GIFTI external binary.
        from_gifti_ext: convert a GIFTI made with this command back into a\
            CIFTI.
        to_nifti: convert to NIFTI1.
        from_nifti: convert a NIFTI (1 or 2) file made with this command back\
            into CIFTI.
        to_text: convert to a plain text file.
        from_text: convert from plain text to cifti.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-convert",
    }
    if to_gifti_ext is not None:
        params["to-gifti-ext"] = to_gifti_ext
    if from_gifti_ext is not None:
        params["from-gifti-ext"] = from_gifti_ext
    if to_nifti is not None:
        params["to-nifti"] = to_nifti
    if from_nifti is not None:
        params["from-nifti"] = from_nifti
    if to_text is not None:
        params["to-text"] = to_text
    if from_text is not None:
        params["from-text"] = from_text
    return params


def cifti_convert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiConvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("to-gifti-ext", None) is not None:
        cifti_convert_to_gifti_ext_validate(params["to-gifti-ext"])
    if params.get("from-gifti-ext", None) is not None:
        cifti_convert_from_gifti_ext_validate(params["from-gifti-ext"])
    if params.get("to-nifti", None) is not None:
        cifti_convert_to_nifti_validate(params["to-nifti"])
    if params.get("from-nifti", None) is not None:
        cifti_convert_from_nifti_validate(params["from-nifti"])
    if params.get("to-text", None) is not None:
        cifti_convert_to_text_validate(params["to-text"])
    if params.get("from-text", None) is not None:
        cifti_convert_from_text_validate(params["from-text"])


def cifti_convert_cargs(
    params: CiftiConvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-convert"
    ])
    if params.get("to-gifti-ext", None) is not None or params.get("from-gifti-ext", None) is not None or params.get("to-nifti", None) is not None or params.get("from-nifti", None) is not None or params.get("to-text", None) is not None or params.get("from-text", None) is not None:
        cargs.extend([
            *(cifti_convert_to_gifti_ext_cargs(params.get("to-gifti-ext", None), execution) if (params.get("to-gifti-ext", None) is not None) else []),
            *(cifti_convert_from_gifti_ext_cargs(params.get("from-gifti-ext", None), execution) if (params.get("from-gifti-ext", None) is not None) else []),
            *(cifti_convert_to_nifti_cargs(params.get("to-nifti", None), execution) if (params.get("to-nifti", None) is not None) else []),
            *(cifti_convert_from_nifti_cargs(params.get("from-nifti", None), execution) if (params.get("from-nifti", None) is not None) else []),
            *(cifti_convert_to_text_cargs(params.get("to-text", None), execution) if (params.get("to-text", None) is not None) else []),
            *(cifti_convert_from_text_cargs(params.get("from-text", None), execution) if (params.get("from-text", None) is not None) else [])
        ])
    return cargs


def cifti_convert_outputs(
    params: CiftiConvertParamsDict,
    execution: Execution,
) -> CiftiConvertOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiConvertOutputs(
        root=execution.output_file("."),
        from_gifti_ext=cifti_convert_from_gifti_ext_outputs(params.get("from-gifti-ext"), execution) if params.get("from-gifti-ext") else None,
        to_nifti=cifti_convert_to_nifti_outputs(params.get("to-nifti"), execution) if params.get("to-nifti") else None,
        from_nifti=cifti_convert_from_nifti_outputs(params.get("from-nifti"), execution) if params.get("from-nifti") else None,
        from_text=cifti_convert_from_text_outputs(params.get("from-text"), execution) if params.get("from-text") else None,
    )
    return ret


def cifti_convert_execute(
    params: CiftiConvertParamsDict,
    runner: Runner | None = None,
) -> CiftiConvertOutputs:
    """
    DUMP CIFTI MATRIX INTO OTHER FORMATS.
    
    This command is used to convert a full CIFTI matrix to/from formats that can
    be used by programs that don't understand CIFTI. You must specify exactly
    one of -to-gifti-ext, -from-gifti-ext, -to-nifti, -from-nifti, -to-text, or
    -from-text.
    
    This command cannot map surface-based parts of the cifti file to a
    spatially-correct volume file, or map volume-based data to the surface, see
    -volume-to-surface-mapping and -metric-to-volume-mapping instead (other
    commands such as -cifti-separate are also required).
    
    If you want to write an existing CIFTI file with a different CIFTI version,
    see -file-convert, and its -cifti-version-convert option.
    
    If you want part of the CIFTI file as a metric, label, or volume file, see
    -cifti-separate. If you want to create a CIFTI file from metric and/or
    volume files, see the -cifti-create-* commands.
    
    If you want to import a matrix in non-CIFTI format that is restricted to an
    ROI, first obtain a template CIFTI file matching that ROI (you can use
    -cifti-create-dense-scalar if you don't have such a file). Use
    -cifti-convert to import it to CIFTI format, and you can then expand the
    file into a standard brainordinates space with
    -cifti-create-dense-from-template. If you want to export only part of a
    CIFTI file, first create an roi-restricted CIFTI file with
    -cifti-restrict-dense-mapping.
    
    The -transpose option to -from-gifti-ext is needed if the replacement binary
    file is in column-major order.
    
    The -unit options accept these values:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiConvertOutputs`).
    """
    cifti_convert_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CONVERT_METADATA)
    params = execution.params(params)
    cargs = cifti_convert_cargs(params, execution)
    ret = cifti_convert_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_convert(
    to_gifti_ext: CiftiConvertToGiftiExtParamsDict | None = None,
    from_gifti_ext: CiftiConvertFromGiftiExtParamsDict | None = None,
    to_nifti: CiftiConvertToNiftiParamsDict | None = None,
    from_nifti: CiftiConvertFromNiftiParamsDict | None = None,
    to_text: CiftiConvertToTextParamsDict | None = None,
    from_text: CiftiConvertFromTextParamsDict | None = None,
    runner: Runner | None = None,
) -> CiftiConvertOutputs:
    """
    DUMP CIFTI MATRIX INTO OTHER FORMATS.
    
    This command is used to convert a full CIFTI matrix to/from formats that can
    be used by programs that don't understand CIFTI. You must specify exactly
    one of -to-gifti-ext, -from-gifti-ext, -to-nifti, -from-nifti, -to-text, or
    -from-text.
    
    This command cannot map surface-based parts of the cifti file to a
    spatially-correct volume file, or map volume-based data to the surface, see
    -volume-to-surface-mapping and -metric-to-volume-mapping instead (other
    commands such as -cifti-separate are also required).
    
    If you want to write an existing CIFTI file with a different CIFTI version,
    see -file-convert, and its -cifti-version-convert option.
    
    If you want part of the CIFTI file as a metric, label, or volume file, see
    -cifti-separate. If you want to create a CIFTI file from metric and/or
    volume files, see the -cifti-create-* commands.
    
    If you want to import a matrix in non-CIFTI format that is restricted to an
    ROI, first obtain a template CIFTI file matching that ROI (you can use
    -cifti-create-dense-scalar if you don't have such a file). Use
    -cifti-convert to import it to CIFTI format, and you can then expand the
    file into a standard brainordinates space with
    -cifti-create-dense-from-template. If you want to export only part of a
    CIFTI file, first create an roi-restricted CIFTI file with
    -cifti-restrict-dense-mapping.
    
    The -transpose option to -from-gifti-ext is needed if the replacement binary
    file is in column-major order.
    
    The -unit options accept these values:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Args:
        to_gifti_ext: convert to GIFTI external binary.
        from_gifti_ext: convert a GIFTI made with this command back into a\
            CIFTI.
        to_nifti: convert to NIFTI1.
        from_nifti: convert a NIFTI (1 or 2) file made with this command back\
            into CIFTI.
        to_text: convert to a plain text file.
        from_text: convert from plain text to cifti.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiConvertOutputs`).
    """
    params = cifti_convert_params(
        to_gifti_ext=to_gifti_ext,
        from_gifti_ext=from_gifti_ext,
        to_nifti=to_nifti,
        from_nifti=from_nifti,
        to_text=to_text,
        from_text=from_text,
    )
    return cifti_convert_execute(params, runner)


__all__ = [
    "CIFTI_CONVERT_METADATA",
    "CiftiConvertFromGiftiExtOutputs",
    "CiftiConvertFromGiftiExtParamsDict",
    "CiftiConvertFromGiftiExtParamsDictTagged",
    "CiftiConvertFromNiftiOutputs",
    "CiftiConvertFromNiftiParamsDict",
    "CiftiConvertFromNiftiParamsDictTagged",
    "CiftiConvertFromTextOutputs",
    "CiftiConvertFromTextParamsDict",
    "CiftiConvertFromTextParamsDictTagged",
    "CiftiConvertOutputs",
    "CiftiConvertParamsDict",
    "CiftiConvertParamsDictTagged",
    "CiftiConvertReplaceBinaryParamsDict",
    "CiftiConvertReplaceBinaryParamsDictTagged",
    "CiftiConvertResetTimepointsParamsDict",
    "CiftiConvertResetTimepointsParamsDictTagged",
    "CiftiConvertResetTimepointsParamsDictTagged_",
    "CiftiConvertResetTimepointsParamsDictTagged_2",
    "CiftiConvertResetTimepointsParamsDict_",
    "CiftiConvertResetTimepointsParamsDict_2",
    "CiftiConvertToGiftiExtParamsDict",
    "CiftiConvertToGiftiExtParamsDictTagged",
    "CiftiConvertToNiftiOutputs",
    "CiftiConvertToNiftiParamsDict",
    "CiftiConvertToNiftiParamsDictTagged",
    "CiftiConvertToTextParamsDict",
    "CiftiConvertToTextParamsDictTagged",
    "cifti_convert",
    "cifti_convert_execute",
    "cifti_convert_from_gifti_ext",
    "cifti_convert_from_nifti",
    "cifti_convert_from_text",
    "cifti_convert_params",
    "cifti_convert_replace_binary",
    "cifti_convert_reset_timepoints",
    "cifti_convert_reset_timepoints_",
    "cifti_convert_reset_timepoints_2",
    "cifti_convert_to_gifti_ext",
    "cifti_convert_to_nifti",
    "cifti_convert_to_text",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_MODIFY_SPHERE_METADATA = Metadata(
    id="bd702111cb8b1766b63c379235de453d7fec39d5.workbench",
    name="surface-modify-sphere",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceModifySphereParamsDictNoTag = typing.TypedDict('_SurfaceModifySphereParamsDictNoTag', {
    "sphere-in": InputPathType,
    "radius": float,
    "sphere-out": str,
    "recenter": bool,
})
SurfaceModifySphereParamsDictTagged = typing.TypedDict('SurfaceModifySphereParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-modify-sphere"],
    "sphere-in": InputPathType,
    "radius": float,
    "sphere-out": str,
    "recenter": bool,
})
SurfaceModifySphereParamsDict = _SurfaceModifySphereParamsDictNoTag | SurfaceModifySphereParamsDictTagged


class SurfaceModifySphereOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceModifySphereParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    sphere_out: OutputPathType
    """the output sphere"""


def surface_modify_sphere_params(
    sphere_in: InputPathType,
    radius: float,
    sphere_out: str,
    recenter: bool = False,
) -> SurfaceModifySphereParamsDictTagged:
    """
    Build parameters.
    
    Args:
        sphere_in: the sphere to modify.
        radius: the radius the output sphere should have.
        sphere_out: the output sphere.
        recenter: recenter the sphere by means of the bounding box.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-modify-sphere",
        "sphere-in": sphere_in,
        "radius": radius,
        "sphere-out": sphere_out,
        "recenter": recenter,
    }
    return params


def surface_modify_sphere_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceModifySphereParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("sphere-in", None) is None:
        raise StyxValidationError("`sphere-in` must not be None")
    if not isinstance(params["sphere-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`sphere-in` has the wrong type: Received `{type(params.get("sphere-in", None))}` expected `InputPathType`')
    if params.get("radius", None) is None:
        raise StyxValidationError("`radius` must not be None")
    if not isinstance(params["radius"], (float, int)):
        raise StyxValidationError(f'`radius` has the wrong type: Received `{type(params.get("radius", None))}` expected `float`')
    if params.get("sphere-out", None) is None:
        raise StyxValidationError("`sphere-out` must not be None")
    if not isinstance(params["sphere-out"], str):
        raise StyxValidationError(f'`sphere-out` has the wrong type: Received `{type(params.get("sphere-out", None))}` expected `str`')
    if params.get("recenter", False) is None:
        raise StyxValidationError("`recenter` must not be None")
    if not isinstance(params["recenter"], bool):
        raise StyxValidationError(f'`recenter` has the wrong type: Received `{type(params.get("recenter", False))}` expected `bool`')


def surface_modify_sphere_cargs(
    params: SurfaceModifySphereParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-modify-sphere"
    ])
    cargs.append(execution.input_file(params.get("sphere-in", None)))
    cargs.append(str(params.get("radius", None)))
    cargs.append(params.get("sphere-out", None))
    if params.get("recenter", False):
        cargs.append("-recenter")
    return cargs


def surface_modify_sphere_outputs(
    params: SurfaceModifySphereParamsDict,
    execution: Execution,
) -> SurfaceModifySphereOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceModifySphereOutputs(
        root=execution.output_file("."),
        sphere_out=execution.output_file(params.get("sphere-out", None)),
    )
    return ret


def surface_modify_sphere_execute(
    params: SurfaceModifySphereParamsDict,
    runner: Runner | None = None,
) -> SurfaceModifySphereOutputs:
    """
    CHANGE RADIUS AND OPTIONALLY RECENTER A SPHERE.
    
    This command may be useful if you have used -surface-resample to resample a
    sphere, which can suffer from problems generally not present in
    -surface-sphere-project-unproject. If the sphere should already be centered
    around the origin, using -recenter may still shift it slightly before
    changing the radius, which is likely to be undesireable.
    
    If <sphere-in> is not close to spherical, or not centered around the origin
    and -recenter is not used, a warning is printed.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceModifySphereOutputs`).
    """
    surface_modify_sphere_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_MODIFY_SPHERE_METADATA)
    params = execution.params(params)
    cargs = surface_modify_sphere_cargs(params, execution)
    ret = surface_modify_sphere_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_modify_sphere(
    sphere_in: InputPathType,
    radius: float,
    sphere_out: str,
    recenter: bool = False,
    runner: Runner | None = None,
) -> SurfaceModifySphereOutputs:
    """
    CHANGE RADIUS AND OPTIONALLY RECENTER A SPHERE.
    
    This command may be useful if you have used -surface-resample to resample a
    sphere, which can suffer from problems generally not present in
    -surface-sphere-project-unproject. If the sphere should already be centered
    around the origin, using -recenter may still shift it slightly before
    changing the radius, which is likely to be undesireable.
    
    If <sphere-in> is not close to spherical, or not centered around the origin
    and -recenter is not used, a warning is printed.
    
    Args:
        sphere_in: the sphere to modify.
        radius: the radius the output sphere should have.
        sphere_out: the output sphere.
        recenter: recenter the sphere by means of the bounding box.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceModifySphereOutputs`).
    """
    params = surface_modify_sphere_params(
        sphere_in=sphere_in,
        radius=radius,
        sphere_out=sphere_out,
        recenter=recenter,
    )
    return surface_modify_sphere_execute(params, runner)


__all__ = [
    "SURFACE_MODIFY_SPHERE_METADATA",
    "SurfaceModifySphereOutputs",
    "SurfaceModifySphereParamsDict",
    "SurfaceModifySphereParamsDictTagged",
    "surface_modify_sphere",
    "surface_modify_sphere_execute",
    "surface_modify_sphere_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_REDUCE_METADATA = Metadata(
    id="640fab587d02e7f004bb3968f135463a35a2774a.workbench",
    name="volume-reduce",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeReduceExcludeOutliersParamsDictNoTag = typing.TypedDict('_VolumeReduceExcludeOutliersParamsDictNoTag', {
    "sigma-below": float,
    "sigma-above": float,
})
VolumeReduceExcludeOutliersParamsDictTagged = typing.TypedDict('VolumeReduceExcludeOutliersParamsDictTagged', {
    "@type": typing.Literal["exclude-outliers"],
    "sigma-below": float,
    "sigma-above": float,
})
VolumeReduceExcludeOutliersParamsDict = _VolumeReduceExcludeOutliersParamsDictNoTag | VolumeReduceExcludeOutliersParamsDictTagged


_VolumeReduceParamsDictNoTag = typing.TypedDict('_VolumeReduceParamsDictNoTag', {
    "volume-in": InputPathType,
    "operation": str,
    "volume-out": str,
    "exclude-outliers": typing.NotRequired[VolumeReduceExcludeOutliersParamsDict | None],
    "only-numeric": bool,
})
VolumeReduceParamsDictTagged = typing.TypedDict('VolumeReduceParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-reduce"],
    "volume-in": InputPathType,
    "operation": str,
    "volume-out": str,
    "exclude-outliers": typing.NotRequired[VolumeReduceExcludeOutliersParamsDict | None],
    "only-numeric": bool,
})
VolumeReduceParamsDict = _VolumeReduceParamsDictNoTag | VolumeReduceParamsDictTagged


def volume_reduce_exclude_outliers(
    sigma_below: float,
    sigma_above: float,
) -> VolumeReduceExcludeOutliersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        sigma_below: number of standard deviations below the mean to include.
        sigma_above: number of standard deviations above the mean to include.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "exclude-outliers",
        "sigma-below": sigma_below,
        "sigma-above": sigma_above,
    }
    return params


def volume_reduce_exclude_outliers_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeReduceExcludeOutliersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("sigma-below", None) is None:
        raise StyxValidationError("`sigma-below` must not be None")
    if not isinstance(params["sigma-below"], (float, int)):
        raise StyxValidationError(f'`sigma-below` has the wrong type: Received `{type(params.get("sigma-below", None))}` expected `float`')
    if params.get("sigma-above", None) is None:
        raise StyxValidationError("`sigma-above` must not be None")
    if not isinstance(params["sigma-above"], (float, int)):
        raise StyxValidationError(f'`sigma-above` has the wrong type: Received `{type(params.get("sigma-above", None))}` expected `float`')


def volume_reduce_exclude_outliers_cargs(
    params: VolumeReduceExcludeOutliersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-exclude-outliers",
        str(params.get("sigma-below", None)),
        str(params.get("sigma-above", None))
    ])
    return cargs


class VolumeReduceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeReduceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_reduce_params(
    volume_in: InputPathType,
    operation: str,
    volume_out: str,
    exclude_outliers: VolumeReduceExcludeOutliersParamsDict | None = None,
    only_numeric: bool = False,
) -> VolumeReduceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the volume file to reduce.
        operation: the reduction operator to use.
        volume_out: the output volume.
        exclude_outliers: exclude non-numeric values and outliers by standard\
            deviation.
        only_numeric: exclude non-numeric values.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-reduce",
        "volume-in": volume_in,
        "operation": operation,
        "volume-out": volume_out,
        "only-numeric": only_numeric,
    }
    if exclude_outliers is not None:
        params["exclude-outliers"] = exclude_outliers
    return params


def volume_reduce_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeReduceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("operation", None) is None:
        raise StyxValidationError("`operation` must not be None")
    if not isinstance(params["operation"], str):
        raise StyxValidationError(f'`operation` has the wrong type: Received `{type(params.get("operation", None))}` expected `str`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("exclude-outliers", None) is not None:
        volume_reduce_exclude_outliers_validate(params["exclude-outliers"])
    if params.get("only-numeric", False) is None:
        raise StyxValidationError("`only-numeric` must not be None")
    if not isinstance(params["only-numeric"], bool):
        raise StyxValidationError(f'`only-numeric` has the wrong type: Received `{type(params.get("only-numeric", False))}` expected `bool`')


def volume_reduce_cargs(
    params: VolumeReduceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-reduce"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(params.get("operation", None))
    cargs.append(params.get("volume-out", None))
    if params.get("exclude-outliers", None) is not None:
        cargs.extend(volume_reduce_exclude_outliers_cargs(params.get("exclude-outliers", None), execution))
    if params.get("only-numeric", False):
        cargs.append("-only-numeric")
    return cargs


def volume_reduce_outputs(
    params: VolumeReduceParamsDict,
    execution: Execution,
) -> VolumeReduceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeReduceOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_reduce_execute(
    params: VolumeReduceParamsDict,
    runner: Runner | None = None,
) -> VolumeReduceOutputs:
    """
    PERFORM REDUCTION OPERATION ACROSS SUBVOLUMES.
    
    For each voxel, takes the data across subvolumes as a vector, and performs
    the specified reduction on it, putting the result into the single output
    volume at that voxel. The reduction operators are as follows:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeReduceOutputs`).
    """
    volume_reduce_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_REDUCE_METADATA)
    params = execution.params(params)
    cargs = volume_reduce_cargs(params, execution)
    ret = volume_reduce_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_reduce(
    volume_in: InputPathType,
    operation: str,
    volume_out: str,
    exclude_outliers: VolumeReduceExcludeOutliersParamsDict | None = None,
    only_numeric: bool = False,
    runner: Runner | None = None,
) -> VolumeReduceOutputs:
    """
    PERFORM REDUCTION OPERATION ACROSS SUBVOLUMES.
    
    For each voxel, takes the data across subvolumes as a vector, and performs
    the specified reduction on it, putting the result into the single output
    volume at that voxel. The reduction operators are as follows:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    .
    
    Args:
        volume_in: the volume file to reduce.
        operation: the reduction operator to use.
        volume_out: the output volume.
        exclude_outliers: exclude non-numeric values and outliers by standard\
            deviation.
        only_numeric: exclude non-numeric values.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeReduceOutputs`).
    """
    params = volume_reduce_params(
        volume_in=volume_in,
        operation=operation,
        volume_out=volume_out,
        exclude_outliers=exclude_outliers,
        only_numeric=only_numeric,
    )
    return volume_reduce_execute(params, runner)


__all__ = [
    "VOLUME_REDUCE_METADATA",
    "VolumeReduceExcludeOutliersParamsDict",
    "VolumeReduceExcludeOutliersParamsDictTagged",
    "VolumeReduceOutputs",
    "VolumeReduceParamsDict",
    "VolumeReduceParamsDictTagged",
    "volume_reduce",
    "volume_reduce_exclude_outliers",
    "volume_reduce_execute",
    "volume_reduce_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_DISTORTION_METADATA = Metadata(
    id="6b557cde1c22a8589b67dd2c10b7db6b8b4df56a.workbench",
    name="surface-distortion",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceDistortionSmoothParamsDictNoTag = typing.TypedDict('_SurfaceDistortionSmoothParamsDictNoTag', {
    "sigma": float,
    "fwhm": bool,
})
SurfaceDistortionSmoothParamsDictTagged = typing.TypedDict('SurfaceDistortionSmoothParamsDictTagged', {
    "@type": typing.Literal["smooth"],
    "sigma": float,
    "fwhm": bool,
})
SurfaceDistortionSmoothParamsDict = _SurfaceDistortionSmoothParamsDictNoTag | SurfaceDistortionSmoothParamsDictTagged


_SurfaceDistortionMatchSurfaceAreaParamsDictNoTag = typing.TypedDict('_SurfaceDistortionMatchSurfaceAreaParamsDictNoTag', {
    "roi-metric": typing.NotRequired[InputPathType | None],
})
SurfaceDistortionMatchSurfaceAreaParamsDictTagged = typing.TypedDict('SurfaceDistortionMatchSurfaceAreaParamsDictTagged', {
    "@type": typing.Literal["match-surface-area"],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
SurfaceDistortionMatchSurfaceAreaParamsDict = _SurfaceDistortionMatchSurfaceAreaParamsDictNoTag | SurfaceDistortionMatchSurfaceAreaParamsDictTagged


_SurfaceDistortionParamsDictNoTag = typing.TypedDict('_SurfaceDistortionParamsDictNoTag', {
    "surface-reference": InputPathType,
    "surface-distorted": InputPathType,
    "metric-out": str,
    "smooth": typing.NotRequired[SurfaceDistortionSmoothParamsDict | None],
    "match-surface-area": typing.NotRequired[SurfaceDistortionMatchSurfaceAreaParamsDict | None],
    "log2": typing.NotRequired[bool | None],
    "edge-method": bool,
    "caret5-method": bool,
})
SurfaceDistortionParamsDictTagged = typing.TypedDict('SurfaceDistortionParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-distortion"],
    "surface-reference": InputPathType,
    "surface-distorted": InputPathType,
    "metric-out": str,
    "smooth": typing.NotRequired[SurfaceDistortionSmoothParamsDict | None],
    "match-surface-area": typing.NotRequired[SurfaceDistortionMatchSurfaceAreaParamsDict | None],
    "log2": typing.NotRequired[bool | None],
    "edge-method": bool,
    "caret5-method": bool,
})
SurfaceDistortionParamsDict = _SurfaceDistortionParamsDictNoTag | SurfaceDistortionParamsDictTagged


def surface_distortion_smooth(
    sigma: float,
    fwhm: bool = False,
) -> SurfaceDistortionSmoothParamsDictTagged:
    """
    Build parameters.
    
    Args:
        sigma: the size of the smoothing kernel in mm, as sigma by default.
        fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "smooth",
        "sigma": sigma,
        "fwhm": fwhm,
    }
    return params


def surface_distortion_smooth_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceDistortionSmoothParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("sigma", None) is None:
        raise StyxValidationError("`sigma` must not be None")
    if not isinstance(params["sigma"], (float, int)):
        raise StyxValidationError(f'`sigma` has the wrong type: Received `{type(params.get("sigma", None))}` expected `float`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def surface_distortion_smooth_cargs(
    params: SurfaceDistortionSmoothParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-smooth",
        str(params.get("sigma", None))
    ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def surface_distortion_match_surface_area(
    roi_metric: InputPathType | None = None,
) -> SurfaceDistortionMatchSurfaceAreaParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_metric: only use the surface area within a given ROI (e.g., to\
            exclude the medial wall)\
            \
            the ROI to use, as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "match-surface-area",
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def surface_distortion_match_surface_area_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceDistortionMatchSurfaceAreaParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def surface_distortion_match_surface_area_cargs(
    params: SurfaceDistortionMatchSurfaceAreaParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-match-surface-area")
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


class SurfaceDistortionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceDistortionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output distortion metric"""


def surface_distortion_params(
    surface_reference: InputPathType,
    surface_distorted: InputPathType,
    metric_out: str,
    smooth: SurfaceDistortionSmoothParamsDict | None = None,
    match_surface_area: SurfaceDistortionMatchSurfaceAreaParamsDict | None = None,
    log2: bool | None = None,
    edge_method: bool = False,
    caret5_method: bool = False,
) -> SurfaceDistortionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_reference: the reference surface.
        surface_distorted: the distorted surface.
        metric_out: the output distortion metric.
        smooth: smooth the area data.
        match_surface_area: isotropically rescale the distorted surface so that\
            it has the same surface area as the reference surface.
        log2: calculate distortion by the local affines between triangles\
            \
            apply base-2 log transform.
        edge_method: calculate distortion of edge lengths rather than areas.
        caret5_method: use the surface distortion method from caret5.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-distortion",
        "surface-reference": surface_reference,
        "surface-distorted": surface_distorted,
        "metric-out": metric_out,
        "edge-method": edge_method,
        "caret5-method": caret5_method,
    }
    if smooth is not None:
        params["smooth"] = smooth
    if match_surface_area is not None:
        params["match-surface-area"] = match_surface_area
    if log2 is not None:
        params["log2"] = log2
    return params


def surface_distortion_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceDistortionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface-reference", None) is None:
        raise StyxValidationError("`surface-reference` must not be None")
    if not isinstance(params["surface-reference"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-reference` has the wrong type: Received `{type(params.get("surface-reference", None))}` expected `InputPathType`')
    if params.get("surface-distorted", None) is None:
        raise StyxValidationError("`surface-distorted` must not be None")
    if not isinstance(params["surface-distorted"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-distorted` has the wrong type: Received `{type(params.get("surface-distorted", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("smooth", None) is not None:
        surface_distortion_smooth_validate(params["smooth"])
    if params.get("match-surface-area", None) is not None:
        surface_distortion_match_surface_area_validate(params["match-surface-area"])
    if params.get("log2", None) is not None:
        if not isinstance(params["log2"], bool):
            raise StyxValidationError(f'`log2` has the wrong type: Received `{type(params.get("log2", None))}` expected `bool | None`')
    if params.get("edge-method", False) is None:
        raise StyxValidationError("`edge-method` must not be None")
    if not isinstance(params["edge-method"], bool):
        raise StyxValidationError(f'`edge-method` has the wrong type: Received `{type(params.get("edge-method", False))}` expected `bool`')
    if params.get("caret5-method", False) is None:
        raise StyxValidationError("`caret5-method` must not be None")
    if not isinstance(params["caret5-method"], bool):
        raise StyxValidationError(f'`caret5-method` has the wrong type: Received `{type(params.get("caret5-method", False))}` expected `bool`')


def surface_distortion_cargs(
    params: SurfaceDistortionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-distortion"
    ])
    cargs.append(execution.input_file(params.get("surface-reference", None)))
    cargs.append(execution.input_file(params.get("surface-distorted", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("smooth", None) is not None or params.get("match-surface-area", None) is not None:
        cargs.extend([
            *(surface_distortion_smooth_cargs(params.get("smooth", None), execution) if (params.get("smooth", None) is not None) else []),
            *(surface_distortion_match_surface_area_cargs(params.get("match-surface-area", None), execution) if (params.get("match-surface-area", None) is not None) else [])
        ])
    if params.get("log2", None) is not None:
        cargs.extend([
            "-local-affine-method",
            "-log2"
        ])
    if params.get("edge-method", False):
        cargs.append("-edge-method")
    if params.get("caret5-method", False):
        cargs.append("-caret5-method")
    return cargs


def surface_distortion_outputs(
    params: SurfaceDistortionParamsDict,
    execution: Execution,
) -> SurfaceDistortionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceDistortionOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def surface_distortion_execute(
    params: SurfaceDistortionParamsDict,
    runner: Runner | None = None,
) -> SurfaceDistortionOutputs:
    """
    MEASURE DISTORTION BETWEEN SURFACES.
    
    This command, when not using -caret5-method, -edge-method, or
    -local-affine-method, is equivalent to using -surface-vertex-areas on each
    surface, smoothing both output metrics with the GEO_GAUSS_EQUAL method on
    the surface they came from if -smooth is specified, and then using the
    formula 'ln(distorted/reference)/ln(2)' on the smoothed results.
    
    When using -caret5-method, it uses the surface distortion method from
    caret5, which takes the base 2 log of the ratio of tile areas, then averages
    those results at each vertex, and then smooths the result on the reference
    surface.
    
    When using -edge-method, the -smooth option is ignored, and the output at
    each vertex is the average of 'abs(ln(refEdge/distortEdge)/ln(2))' over all
    edges connected to the vertex.
    
    When using -local-affine-method, the -smooth option is ignored. The output
    is two columns, the first is the area distortion ratio, and the second is
    anisotropic strain. These are calculated by an affine transform between
    matching triangles, and then averaged across the triangles of a vertex.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceDistortionOutputs`).
    """
    surface_distortion_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_DISTORTION_METADATA)
    params = execution.params(params)
    cargs = surface_distortion_cargs(params, execution)
    ret = surface_distortion_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_distortion(
    surface_reference: InputPathType,
    surface_distorted: InputPathType,
    metric_out: str,
    smooth: SurfaceDistortionSmoothParamsDict | None = None,
    match_surface_area: SurfaceDistortionMatchSurfaceAreaParamsDict | None = None,
    log2: bool | None = None,
    edge_method: bool = False,
    caret5_method: bool = False,
    runner: Runner | None = None,
) -> SurfaceDistortionOutputs:
    """
    MEASURE DISTORTION BETWEEN SURFACES.
    
    This command, when not using -caret5-method, -edge-method, or
    -local-affine-method, is equivalent to using -surface-vertex-areas on each
    surface, smoothing both output metrics with the GEO_GAUSS_EQUAL method on
    the surface they came from if -smooth is specified, and then using the
    formula 'ln(distorted/reference)/ln(2)' on the smoothed results.
    
    When using -caret5-method, it uses the surface distortion method from
    caret5, which takes the base 2 log of the ratio of tile areas, then averages
    those results at each vertex, and then smooths the result on the reference
    surface.
    
    When using -edge-method, the -smooth option is ignored, and the output at
    each vertex is the average of 'abs(ln(refEdge/distortEdge)/ln(2))' over all
    edges connected to the vertex.
    
    When using -local-affine-method, the -smooth option is ignored. The output
    is two columns, the first is the area distortion ratio, and the second is
    anisotropic strain. These are calculated by an affine transform between
    matching triangles, and then averaged across the triangles of a vertex.
    
    Args:
        surface_reference: the reference surface.
        surface_distorted: the distorted surface.
        metric_out: the output distortion metric.
        smooth: smooth the area data.
        match_surface_area: isotropically rescale the distorted surface so that\
            it has the same surface area as the reference surface.
        log2: calculate distortion by the local affines between triangles\
            \
            apply base-2 log transform.
        edge_method: calculate distortion of edge lengths rather than areas.
        caret5_method: use the surface distortion method from caret5.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceDistortionOutputs`).
    """
    params = surface_distortion_params(
        surface_reference=surface_reference,
        surface_distorted=surface_distorted,
        metric_out=metric_out,
        smooth=smooth,
        match_surface_area=match_surface_area,
        log2=log2,
        edge_method=edge_method,
        caret5_method=caret5_method,
    )
    return surface_distortion_execute(params, runner)


__all__ = [
    "SURFACE_DISTORTION_METADATA",
    "SurfaceDistortionMatchSurfaceAreaParamsDict",
    "SurfaceDistortionMatchSurfaceAreaParamsDictTagged",
    "SurfaceDistortionOutputs",
    "SurfaceDistortionParamsDict",
    "SurfaceDistortionParamsDictTagged",
    "SurfaceDistortionSmoothParamsDict",
    "SurfaceDistortionSmoothParamsDictTagged",
    "surface_distortion",
    "surface_distortion_execute",
    "surface_distortion_match_surface_area",
    "surface_distortion_params",
    "surface_distortion_smooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_WEIGHTED_STATS_METADATA = Metadata(
    id="014a1bf95b4718c304bf3aa95e26fa00e131122c.workbench",
    name="cifti-weighted-stats",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiWeightedStatsSpatialWeightsParamsDictNoTag = typing.TypedDict('_CiftiWeightedStatsSpatialWeightsParamsDictNoTag', {
    "cerebellum-metric": typing.NotRequired[InputPathType | None],
    "right-metric": typing.NotRequired[InputPathType | None],
    "left-metric": typing.NotRequired[InputPathType | None],
    "cerebellum-surf": typing.NotRequired[InputPathType | None],
    "right-surf": typing.NotRequired[InputPathType | None],
    "left-surf": typing.NotRequired[InputPathType | None],
})
CiftiWeightedStatsSpatialWeightsParamsDictTagged = typing.TypedDict('CiftiWeightedStatsSpatialWeightsParamsDictTagged', {
    "@type": typing.Literal["spatial-weights"],
    "cerebellum-metric": typing.NotRequired[InputPathType | None],
    "right-metric": typing.NotRequired[InputPathType | None],
    "left-metric": typing.NotRequired[InputPathType | None],
    "cerebellum-surf": typing.NotRequired[InputPathType | None],
    "right-surf": typing.NotRequired[InputPathType | None],
    "left-surf": typing.NotRequired[InputPathType | None],
})
CiftiWeightedStatsSpatialWeightsParamsDict = _CiftiWeightedStatsSpatialWeightsParamsDictNoTag | CiftiWeightedStatsSpatialWeightsParamsDictTagged


_CiftiWeightedStatsRoiParamsDictNoTag = typing.TypedDict('_CiftiWeightedStatsRoiParamsDictNoTag', {
    "roi-cifti": InputPathType,
    "match-maps": bool,
})
CiftiWeightedStatsRoiParamsDictTagged = typing.TypedDict('CiftiWeightedStatsRoiParamsDictTagged', {
    "@type": typing.Literal["roi"],
    "roi-cifti": InputPathType,
    "match-maps": bool,
})
CiftiWeightedStatsRoiParamsDict = _CiftiWeightedStatsRoiParamsDictNoTag | CiftiWeightedStatsRoiParamsDictTagged


_CiftiWeightedStatsParamsDictNoTag = typing.TypedDict('_CiftiWeightedStatsParamsDictNoTag', {
    "cifti-in": InputPathType,
    "spatial-weights": typing.NotRequired[CiftiWeightedStatsSpatialWeightsParamsDict | None],
    "roi": typing.NotRequired[CiftiWeightedStatsRoiParamsDict | None],
    "percent": typing.NotRequired[float | None],
    "sample": typing.NotRequired[bool | None],
    "column": typing.NotRequired[int | None],
    "weight-cifti": typing.NotRequired[InputPathType | None],
    "show-map-name": bool,
    "sum": bool,
    "mean": bool,
})
CiftiWeightedStatsParamsDictTagged = typing.TypedDict('CiftiWeightedStatsParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-weighted-stats"],
    "cifti-in": InputPathType,
    "spatial-weights": typing.NotRequired[CiftiWeightedStatsSpatialWeightsParamsDict | None],
    "roi": typing.NotRequired[CiftiWeightedStatsRoiParamsDict | None],
    "percent": typing.NotRequired[float | None],
    "sample": typing.NotRequired[bool | None],
    "column": typing.NotRequired[int | None],
    "weight-cifti": typing.NotRequired[InputPathType | None],
    "show-map-name": bool,
    "sum": bool,
    "mean": bool,
})
CiftiWeightedStatsParamsDict = _CiftiWeightedStatsParamsDictNoTag | CiftiWeightedStatsParamsDictTagged


def cifti_weighted_stats_spatial_weights(
    cerebellum_metric: InputPathType | None = None,
    right_metric: InputPathType | None = None,
    left_metric: InputPathType | None = None,
    cerebellum_surf: InputPathType | None = None,
    right_surf: InputPathType | None = None,
    left_surf: InputPathType | None = None,
) -> CiftiWeightedStatsSpatialWeightsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cerebellum_metric: use a metric file for cerebellum vertex areas\
            \
            metric file containing cerebellum vertex areas.
        right_metric: use a metric file for right vertex areas\
            \
            metric file containing right vertex areas.
        left_metric: use a metric file for left vertex areas\
            \
            metric file containing left vertex areas.
        cerebellum_surf: use a surface for cerebellum vertex areas\
            \
            the cerebellum surface to use, areas are in mm^2.
        right_surf: use a surface for right vertex areas\
            \
            the right surface to use, areas are in mm^2.
        left_surf: use a surface for left vertex areas\
            \
            the left surface to use, areas are in mm^2.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "spatial-weights",
    }
    if cerebellum_metric is not None:
        params["cerebellum-metric"] = cerebellum_metric
    if right_metric is not None:
        params["right-metric"] = right_metric
    if left_metric is not None:
        params["left-metric"] = left_metric
    if cerebellum_surf is not None:
        params["cerebellum-surf"] = cerebellum_surf
    if right_surf is not None:
        params["right-surf"] = right_surf
    if left_surf is not None:
        params["left-surf"] = left_surf
    return params


def cifti_weighted_stats_spatial_weights_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiWeightedStatsSpatialWeightsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cerebellum-metric", None) is not None:
        if not isinstance(params["cerebellum-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`cerebellum-metric` has the wrong type: Received `{type(params.get("cerebellum-metric", None))}` expected `InputPathType | None`')
    if params.get("right-metric", None) is not None:
        if not isinstance(params["right-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`right-metric` has the wrong type: Received `{type(params.get("right-metric", None))}` expected `InputPathType | None`')
    if params.get("left-metric", None) is not None:
        if not isinstance(params["left-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`left-metric` has the wrong type: Received `{type(params.get("left-metric", None))}` expected `InputPathType | None`')
    if params.get("cerebellum-surf", None) is not None:
        if not isinstance(params["cerebellum-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`cerebellum-surf` has the wrong type: Received `{type(params.get("cerebellum-surf", None))}` expected `InputPathType | None`')
    if params.get("right-surf", None) is not None:
        if not isinstance(params["right-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`right-surf` has the wrong type: Received `{type(params.get("right-surf", None))}` expected `InputPathType | None`')
    if params.get("left-surf", None) is not None:
        if not isinstance(params["left-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`left-surf` has the wrong type: Received `{type(params.get("left-surf", None))}` expected `InputPathType | None`')


def cifti_weighted_stats_spatial_weights_cargs(
    params: CiftiWeightedStatsSpatialWeightsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-spatial-weights")
    if params.get("cerebellum-metric", None) is not None:
        cargs.extend([
            "-cerebellum-area-metric",
            execution.input_file(params.get("cerebellum-metric", None))
        ])
    if params.get("right-metric", None) is not None:
        cargs.extend([
            "-right-area-metric",
            execution.input_file(params.get("right-metric", None))
        ])
    if params.get("left-metric", None) is not None:
        cargs.extend([
            "-left-area-metric",
            execution.input_file(params.get("left-metric", None))
        ])
    if params.get("cerebellum-surf", None) is not None:
        cargs.extend([
            "-cerebellum-area-surf",
            execution.input_file(params.get("cerebellum-surf", None))
        ])
    if params.get("right-surf", None) is not None:
        cargs.extend([
            "-right-area-surf",
            execution.input_file(params.get("right-surf", None))
        ])
    if params.get("left-surf", None) is not None:
        cargs.extend([
            "-left-area-surf",
            execution.input_file(params.get("left-surf", None))
        ])
    return cargs


def cifti_weighted_stats_roi(
    roi_cifti: InputPathType,
    match_maps: bool = False,
) -> CiftiWeightedStatsRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_cifti: the roi, as a cifti file.
        match_maps: each column of input uses the corresponding column from the\
            roi file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-cifti": roi_cifti,
        "match-maps": match_maps,
    }
    return params


def cifti_weighted_stats_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiWeightedStatsRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-cifti", None) is None:
        raise StyxValidationError("`roi-cifti` must not be None")
    if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType`')
    if params.get("match-maps", False) is None:
        raise StyxValidationError("`match-maps` must not be None")
    if not isinstance(params["match-maps"], bool):
        raise StyxValidationError(f'`match-maps` has the wrong type: Received `{type(params.get("match-maps", False))}` expected `bool`')


def cifti_weighted_stats_roi_cargs(
    params: CiftiWeightedStatsRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        execution.input_file(params.get("roi-cifti", None))
    ])
    if params.get("match-maps", False):
        cargs.append("-match-maps")
    return cargs


class CiftiWeightedStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiWeightedStatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_weighted_stats_params(
    cifti_in: InputPathType,
    spatial_weights: CiftiWeightedStatsSpatialWeightsParamsDict | None = None,
    roi: CiftiWeightedStatsRoiParamsDict | None = None,
    percent: float | None = None,
    sample: bool | None = None,
    column: int | None = None,
    weight_cifti: InputPathType | None = None,
    show_map_name: bool = False,
    sum_: bool = False,
    mean: bool = False,
) -> CiftiWeightedStatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti.
        spatial_weights: use vertex area and voxel volume as weights.
        roi: only consider data inside an roi.
        percent: compute weighted percentile\
            \
            the percentile to find, must be between 0 and 100.
        sample: compute weighted standard deviation\
            \
            estimate population stdev from the sample.
        column: only display output for one column\
            \
            the column to use (1-based).
        weight_cifti: use a cifti file containing weights\
            \
            the weights to use, as a cifti file.
        show_map_name: print map index and name before each output.
        sum_: compute weighted sum.
        mean: compute weighted mean.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-weighted-stats",
        "cifti-in": cifti_in,
        "show-map-name": show_map_name,
        "sum": sum_,
        "mean": mean,
    }
    if spatial_weights is not None:
        params["spatial-weights"] = spatial_weights
    if roi is not None:
        params["roi"] = roi
    if percent is not None:
        params["percent"] = percent
    if sample is not None:
        params["sample"] = sample
    if column is not None:
        params["column"] = column
    if weight_cifti is not None:
        params["weight-cifti"] = weight_cifti
    return params


def cifti_weighted_stats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiWeightedStatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("spatial-weights", None) is not None:
        cifti_weighted_stats_spatial_weights_validate(params["spatial-weights"])
    if params.get("roi", None) is not None:
        cifti_weighted_stats_roi_validate(params["roi"])
    if params.get("percent", None) is not None:
        if not isinstance(params["percent"], (float, int)):
            raise StyxValidationError(f'`percent` has the wrong type: Received `{type(params.get("percent", None))}` expected `float | None`')
    if params.get("sample", None) is not None:
        if not isinstance(params["sample"], bool):
            raise StyxValidationError(f'`sample` has the wrong type: Received `{type(params.get("sample", None))}` expected `bool | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], int):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `int | None`')
    if params.get("weight-cifti", None) is not None:
        if not isinstance(params["weight-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`weight-cifti` has the wrong type: Received `{type(params.get("weight-cifti", None))}` expected `InputPathType | None`')
    if params.get("show-map-name", False) is None:
        raise StyxValidationError("`show-map-name` must not be None")
    if not isinstance(params["show-map-name"], bool):
        raise StyxValidationError(f'`show-map-name` has the wrong type: Received `{type(params.get("show-map-name", False))}` expected `bool`')
    if params.get("sum", False) is None:
        raise StyxValidationError("`sum` must not be None")
    if not isinstance(params["sum"], bool):
        raise StyxValidationError(f'`sum` has the wrong type: Received `{type(params.get("sum", False))}` expected `bool`')
    if params.get("mean", False) is None:
        raise StyxValidationError("`mean` must not be None")
    if not isinstance(params["mean"], bool):
        raise StyxValidationError(f'`mean` has the wrong type: Received `{type(params.get("mean", False))}` expected `bool`')


def cifti_weighted_stats_cargs(
    params: CiftiWeightedStatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-weighted-stats"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    if params.get("spatial-weights", None) is not None or params.get("roi", None) is not None:
        cargs.extend([
            *(cifti_weighted_stats_spatial_weights_cargs(params.get("spatial-weights", None), execution) if (params.get("spatial-weights", None) is not None) else []),
            *(cifti_weighted_stats_roi_cargs(params.get("roi", None), execution) if (params.get("roi", None) is not None) else [])
        ])
    if params.get("percent", None) is not None:
        cargs.extend([
            "-percentile",
            str(params.get("percent", None))
        ])
    if params.get("sample", None) is not None:
        cargs.extend([
            "-stdev",
            "-sample"
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            str(params.get("column", None))
        ])
    if params.get("weight-cifti", None) is not None:
        cargs.extend([
            "-cifti-weights",
            execution.input_file(params.get("weight-cifti", None))
        ])
    if params.get("show-map-name", False):
        cargs.append("-show-map-name")
    if params.get("sum", False):
        cargs.append("-sum")
    if params.get("mean", False):
        cargs.append("-mean")
    return cargs


def cifti_weighted_stats_outputs(
    params: CiftiWeightedStatsParamsDict,
    execution: Execution,
) -> CiftiWeightedStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiWeightedStatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_weighted_stats_execute(
    params: CiftiWeightedStatsParamsDict,
    runner: Runner | None = None,
) -> CiftiWeightedStatsOutputs:
    """
    WEIGHTED STATISTICS ALONG CIFTI COLUMNS.
    
    If the mapping along column is brain models, for each column of the input,
    the specified operation is done on each surface and across all voxels, and
    the results are printed on separate lines. For other mapping types, the
    operation is done on each column, and one line per map is printed. Exactly
    one of -spatial-weights or -cifti-weights must be specified. Use -column to
    only give output for a single column. If the -roi option is used without
    -match-maps, then each line will contain as many numbers as there are maps
    in the ROI file, separated by tab characters. Exactly one of -mean, -stdev,
    -percentile or -sum must be specified.
    
    Using -sum with -spatial-weights (or with -cifti-weights and a cifti
    containing weights of similar meaning) is equivalent to integrating with
    respect to area and volume. When the input is binary ROIs, this will
    therefore output the area or volume of each ROI.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiWeightedStatsOutputs`).
    """
    cifti_weighted_stats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_WEIGHTED_STATS_METADATA)
    params = execution.params(params)
    cargs = cifti_weighted_stats_cargs(params, execution)
    ret = cifti_weighted_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_weighted_stats(
    cifti_in: InputPathType,
    spatial_weights: CiftiWeightedStatsSpatialWeightsParamsDict | None = None,
    roi: CiftiWeightedStatsRoiParamsDict | None = None,
    percent: float | None = None,
    sample: bool | None = None,
    column: int | None = None,
    weight_cifti: InputPathType | None = None,
    show_map_name: bool = False,
    sum_: bool = False,
    mean: bool = False,
    runner: Runner | None = None,
) -> CiftiWeightedStatsOutputs:
    """
    WEIGHTED STATISTICS ALONG CIFTI COLUMNS.
    
    If the mapping along column is brain models, for each column of the input,
    the specified operation is done on each surface and across all voxels, and
    the results are printed on separate lines. For other mapping types, the
    operation is done on each column, and one line per map is printed. Exactly
    one of -spatial-weights or -cifti-weights must be specified. Use -column to
    only give output for a single column. If the -roi option is used without
    -match-maps, then each line will contain as many numbers as there are maps
    in the ROI file, separated by tab characters. Exactly one of -mean, -stdev,
    -percentile or -sum must be specified.
    
    Using -sum with -spatial-weights (or with -cifti-weights and a cifti
    containing weights of similar meaning) is equivalent to integrating with
    respect to area and volume. When the input is binary ROIs, this will
    therefore output the area or volume of each ROI.
    
    Args:
        cifti_in: the input cifti.
        spatial_weights: use vertex area and voxel volume as weights.
        roi: only consider data inside an roi.
        percent: compute weighted percentile\
            \
            the percentile to find, must be between 0 and 100.
        sample: compute weighted standard deviation\
            \
            estimate population stdev from the sample.
        column: only display output for one column\
            \
            the column to use (1-based).
        weight_cifti: use a cifti file containing weights\
            \
            the weights to use, as a cifti file.
        show_map_name: print map index and name before each output.
        sum_: compute weighted sum.
        mean: compute weighted mean.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiWeightedStatsOutputs`).
    """
    params = cifti_weighted_stats_params(
        cifti_in=cifti_in,
        spatial_weights=spatial_weights,
        roi=roi,
        percent=percent,
        sample=sample,
        column=column,
        weight_cifti=weight_cifti,
        show_map_name=show_map_name,
        sum_=sum_,
        mean=mean,
    )
    return cifti_weighted_stats_execute(params, runner)


__all__ = [
    "CIFTI_WEIGHTED_STATS_METADATA",
    "CiftiWeightedStatsOutputs",
    "CiftiWeightedStatsParamsDict",
    "CiftiWeightedStatsParamsDictTagged",
    "CiftiWeightedStatsRoiParamsDict",
    "CiftiWeightedStatsRoiParamsDictTagged",
    "CiftiWeightedStatsSpatialWeightsParamsDict",
    "CiftiWeightedStatsSpatialWeightsParamsDictTagged",
    "cifti_weighted_stats",
    "cifti_weighted_stats_execute",
    "cifti_weighted_stats_params",
    "cifti_weighted_stats_roi",
    "cifti_weighted_stats_spatial_weights",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_SMOOTHING_METADATA = Metadata(
    id="ed8f3a670c4c85072256a932d9737b524da04f12.workbench",
    name="surface-smoothing",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceSmoothingParamsDictNoTag = typing.TypedDict('_SurfaceSmoothingParamsDictNoTag', {
    "surface-in": InputPathType,
    "smoothing-strength": float,
    "smoothing-iterations": int,
    "surface-out": str,
})
SurfaceSmoothingParamsDictTagged = typing.TypedDict('SurfaceSmoothingParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-smoothing"],
    "surface-in": InputPathType,
    "smoothing-strength": float,
    "smoothing-iterations": int,
    "surface-out": str,
})
SurfaceSmoothingParamsDict = _SurfaceSmoothingParamsDictNoTag | SurfaceSmoothingParamsDictTagged


class SurfaceSmoothingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceSmoothingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """output surface file"""


def surface_smoothing_params(
    surface_in: InputPathType,
    smoothing_strength: float,
    smoothing_iterations: int,
    surface_out: str,
) -> SurfaceSmoothingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_in: the surface file to smooth.
        smoothing_strength: smoothing strength (ranges [0.0 - 1.0]).
        smoothing_iterations: smoothing iterations.
        surface_out: output surface file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-smoothing",
        "surface-in": surface_in,
        "smoothing-strength": smoothing_strength,
        "smoothing-iterations": smoothing_iterations,
        "surface-out": surface_out,
    }
    return params


def surface_smoothing_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceSmoothingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface-in", None) is None:
        raise StyxValidationError("`surface-in` must not be None")
    if not isinstance(params["surface-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-in` has the wrong type: Received `{type(params.get("surface-in", None))}` expected `InputPathType`')
    if params.get("smoothing-strength", None) is None:
        raise StyxValidationError("`smoothing-strength` must not be None")
    if not isinstance(params["smoothing-strength"], (float, int)):
        raise StyxValidationError(f'`smoothing-strength` has the wrong type: Received `{type(params.get("smoothing-strength", None))}` expected `float`')
    if params.get("smoothing-iterations", None) is None:
        raise StyxValidationError("`smoothing-iterations` must not be None")
    if not isinstance(params["smoothing-iterations"], int):
        raise StyxValidationError(f'`smoothing-iterations` has the wrong type: Received `{type(params.get("smoothing-iterations", None))}` expected `int`')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')


def surface_smoothing_cargs(
    params: SurfaceSmoothingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-smoothing"
    ])
    cargs.append(execution.input_file(params.get("surface-in", None)))
    cargs.append(str(params.get("smoothing-strength", None)))
    cargs.append(str(params.get("smoothing-iterations", None)))
    cargs.append(params.get("surface-out", None))
    return cargs


def surface_smoothing_outputs(
    params: SurfaceSmoothingParamsDict,
    execution: Execution,
) -> SurfaceSmoothingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceSmoothingOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
    )
    return ret


def surface_smoothing_execute(
    params: SurfaceSmoothingParamsDict,
    runner: Runner | None = None,
) -> SurfaceSmoothingOutputs:
    """
    SURFACE SMOOTHING.
    
    Smooths a surface by averaging vertex coordinates with those of the
    neighboring vertices.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceSmoothingOutputs`).
    """
    surface_smoothing_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_SMOOTHING_METADATA)
    params = execution.params(params)
    cargs = surface_smoothing_cargs(params, execution)
    ret = surface_smoothing_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_smoothing(
    surface_in: InputPathType,
    smoothing_strength: float,
    smoothing_iterations: int,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceSmoothingOutputs:
    """
    SURFACE SMOOTHING.
    
    Smooths a surface by averaging vertex coordinates with those of the
    neighboring vertices.
    
    Args:
        surface_in: the surface file to smooth.
        smoothing_strength: smoothing strength (ranges [0.0 - 1.0]).
        smoothing_iterations: smoothing iterations.
        surface_out: output surface file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceSmoothingOutputs`).
    """
    params = surface_smoothing_params(
        surface_in=surface_in,
        smoothing_strength=smoothing_strength,
        smoothing_iterations=smoothing_iterations,
        surface_out=surface_out,
    )
    return surface_smoothing_execute(params, runner)


__all__ = [
    "SURFACE_SMOOTHING_METADATA",
    "SurfaceSmoothingOutputs",
    "SurfaceSmoothingParamsDict",
    "SurfaceSmoothingParamsDictTagged",
    "surface_smoothing",
    "surface_smoothing_execute",
    "surface_smoothing_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_REMOVE_ISLANDS_METADATA = Metadata(
    id="e1ca1127c9380dc9d8ce9364a1b74e3cb9676753.workbench",
    name="metric-remove-islands",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricRemoveIslandsParamsDictNoTag = typing.TypedDict('_MetricRemoveIslandsParamsDictNoTag', {
    "surface": InputPathType,
    "metric-in": InputPathType,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
})
MetricRemoveIslandsParamsDictTagged = typing.TypedDict('MetricRemoveIslandsParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-remove-islands"],
    "surface": InputPathType,
    "metric-in": InputPathType,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
})
MetricRemoveIslandsParamsDict = _MetricRemoveIslandsParamsDictNoTag | MetricRemoveIslandsParamsDictTagged


class MetricRemoveIslandsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricRemoveIslandsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output ROI metric"""


def metric_remove_islands_params(
    surface: InputPathType,
    metric_in: InputPathType,
    metric_out: str,
    area_metric: InputPathType | None = None,
) -> MetricRemoveIslandsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to use for neighbor information.
        metric_in: the input ROI metric.
        metric_out: the output ROI metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-remove-islands",
        "surface": surface,
        "metric-in": metric_in,
        "metric-out": metric_out,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def metric_remove_islands_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricRemoveIslandsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def metric_remove_islands_cargs(
    params: MetricRemoveIslandsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-remove-islands"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric-in", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def metric_remove_islands_outputs(
    params: MetricRemoveIslandsParamsDict,
    execution: Execution,
) -> MetricRemoveIslandsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricRemoveIslandsOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_remove_islands_execute(
    params: MetricRemoveIslandsParamsDict,
    runner: Runner | None = None,
) -> MetricRemoveIslandsOutputs:
    """
    REMOVE ISLANDS FROM AN ROI METRIC.
    
    Finds all connected areas in the ROI, and zeros out all but the largest one,
    in terms of surface area.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricRemoveIslandsOutputs`).
    """
    metric_remove_islands_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_REMOVE_ISLANDS_METADATA)
    params = execution.params(params)
    cargs = metric_remove_islands_cargs(params, execution)
    ret = metric_remove_islands_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_remove_islands(
    surface: InputPathType,
    metric_in: InputPathType,
    metric_out: str,
    area_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricRemoveIslandsOutputs:
    """
    REMOVE ISLANDS FROM AN ROI METRIC.
    
    Finds all connected areas in the ROI, and zeros out all but the largest one,
    in terms of surface area.
    
    Args:
        surface: the surface to use for neighbor information.
        metric_in: the input ROI metric.
        metric_out: the output ROI metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricRemoveIslandsOutputs`).
    """
    params = metric_remove_islands_params(
        surface=surface,
        metric_in=metric_in,
        metric_out=metric_out,
        area_metric=area_metric,
    )
    return metric_remove_islands_execute(params, runner)


__all__ = [
    "METRIC_REMOVE_ISLANDS_METADATA",
    "MetricRemoveIslandsOutputs",
    "MetricRemoveIslandsParamsDict",
    "MetricRemoveIslandsParamsDictTagged",
    "metric_remove_islands",
    "metric_remove_islands_execute",
    "metric_remove_islands_params",
]
